/**
 * @file formula.h
 * @author Léo Perrin
 * @date Time-stamp: <2012-06-16 17:14:00 leo>
 */

#ifndef _FORMULA_H
#define _FORMULA_H_

#include <vector>
#include <string>
#include <fstream>

#include "clause.h"
#include "variableset.h"

#define CLAUSE_BUFFER_SIZE 10000

namespace cnf {

/**
 * @class Formula
 * @brief An instance implements a complete CNF formula using several
 * Clause instances.
 *
 * A CNF formula is the conjunction of an arbitrary number of
 * disjunctive clauses. These are represented by Clause instances
 * stored in a std::vector. Note that the codes of the literals of the
 * clauses are given by a VariableSet instance connected to this one.
 *
 * The CNF is printed in the ostream specified at the construction as
 * it is build. For efficiency however, it is printed by bursts of at
 * most CLAUSE_BUFFER_SIZE clauses.
 */
class Formula
{
	private:
		/**
		 * @brief The set of the variables.
		 * 
		 * It "lives" independantly of the formula so it is a pointer
		 * to it.
		 */
		VariableSet * varSet;

		/**
		 * @brief Stores the CNF clauses of this Formula as Clause
		 * instances.
		 */
		std::vector<Clause> cnfs;

		/**
		 * @brief The clause currently being modified. It is
		 * modified by the newClause(), addLit() and
		 * pushClause().
		 */
		Clause bufferClause;
                
                /**
                 * @brief The stream to which the instance must be
                 * printed.
                 */
                std::ostream * output;


	public:
		/**
		 * @brief Creates an instance of Formula which will
		 * use the VariableSet of the input. Creates also the
		 * DIMACS formated file corresponding to this formula.
		 * 
		 * For more info on the DIMACS encoding, see <a href="www.csc.
		 * kth.se/~jakobn/teaching/proofcplx11/minisat.php">this</a>. A
		 * commentary will be added in the beginning of the file.
		 *
		 * @param out A pointer to the stream where the file must be
		 * written.
		 * @param comment The commentary to add on the first line.
		 * @param S A pointer to the VariableSet to link to this Formula.
		*/
		Formula(VariableSet*, std::ostream*, std::string);

                /**
                 * @brief When this instance is destroyed, prints the
                 * clauses still in the cnfs attribute.
                 */
                ~Formula();

		/**
		 * @brief Reinitialise the bufferClause attribute to
		 * an emtpy one.
		 */
		void newClause();

		/**
		 * @brief Adds a new variable to the bufferClause
		 * attribute, negated if sign==false.
                 * 
                 * @param s The subset in which to take the variable
                 * @param sign False if the literal is negated, true otherwise.
                 * @param i First coordinate.
                 * @param j Second coordinate.
		 */
		void addLit(unsigned int, bool, unsigned int, unsigned int);

		/**
		 * @brief Adds a new variable to the bufferClause
		 * attribute, negated if sign==false
                 *
                 * @param s The subset in which to take the variable
                 * @param sign False if the literal is negated, true otherwise.
                 * @param i First coordinate.
                 * @param j Second coordinate.
                 * @param k Third coordinate.
		 */
		void addLit(unsigned int, bool,
                            unsigned int, unsigned int, unsigned int);

                /**
                 * @brief Adds a new variable to the bufferClause
                 * attribute from its code.
                 * 
                 * @param c The code of the variable to add to the
                 * clause.
                 */
                void addLitCode(int);

		/**
		 * @brief Use this method to put the current clause in
		 * the formula modeled by this instance.
		 */
		void pushClause();

		/**
		 * @brief Prints the content of the cnfs attribute to
		 * the output and reinitialises it.
                 */
		void flushCNF();
};


} // closing namespace

#endif
